open OUnit
open Schema
open Indexes
open Regstab

let parsed s =
  let lexbuf = Lexing.from_string s in
  Parser.main Lexer.token lexbuf

open Schema_test.Values

let (sch1,_)         = parsed "(\\/i=1..n P_i) /\\ (/\\i=1..n ~P_i)"
let (sch2,cstr2)     = parsed "P_1 /\\ /\\i=1..n (~P_i \\/ P_i+1) /\\ ~P_n | n >= 1"
let (sum_i,_)        = parsed "S_i   <-> (A_i (+) (B_i (+) C_i))"
let (carry_i,_)      = parsed "C_i+1 <-> ((A_i /\\ B_i) \\/ (C_i /\\ A_i) \\/ (C_i /\\ B_i))"
let (null_carry_1,_) = parsed "~C_1"
let adder            = conj null_carry_1 (it_conj vari int1 termvarn (conj sum_i carry_i))
let (nullB,_)        = parsed "/\\i=1..n ~B_i"
let (conc,_)         = parsed "\\/i=1..n (A_i (+) S_i)"
let add0             = conj adder (conj nullB conc)
let add0'            = conj adder nullB

let test =
  "regstab" >:::
    [
      "regstab" >::
        (fun () ->
          "/\\i=1..n ~P_i /\\ \\/i=1..n P_i"                     @? (regstab sch1              = Satisfiability.NoModelFound);
          "P_1 /\\ /\\i=1..n (~P_i \\/ P_i+1) /\\ ~P_n | n >= 1" @? (regstab ~cstr:cstr2 sch2  = Satisfiability.NoModelFound);
          "adder"                                                @? (regstab ~cstr:cstr2 add0  = Satisfiability.NoModelFound);
          "bad adder1"                                           @? (regstab ~cstr:cstr2 add0' = Satisfiability.ModelFound);
          "bad adder2"                                           @? (regstab ~cstr:cstr2 adder = Satisfiability.ModelFound);
          "bad adder3"                                           @? (regstab ~cstr:cstr2 conc  = Satisfiability.ModelFound);
        );
      Propositional_test.parametrized_test regstab;
    ]
